## $Id: student.py 7677 2012-02-22 10:35:47Z henrik $
##
## Copyright (C) 2011 Uli Fouquet & Henrik Bettermann
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
##
"""
Container for the various objects owned by students.
"""
import os
import grok
from zope.component import getUtility
from zope.component.interfaces import IFactory
from zope.interface import implementedBy
from hurry.workflow.interfaces import IWorkflowState, IWorkflowInfo
from zope.securitypolicy.interfaces import IPrincipalRoleManager
from waeup.sirp.interfaces import (
    IObjectHistory, IUserAccount, IFileStoreNameChooser, IFileStoreHandler,
    ISIRPUtils, CLEARANCE, registration_states_vocab)
from waeup.sirp.image import SIRPImageFile
from waeup.sirp.imagestorage import DefaultFileStoreHandler
from waeup.sirp.students.interfaces import IStudent, IStudentNavigation
from waeup.sirp.students.studycourse import StudentStudyCourse
from waeup.sirp.students.payments import StudentPaymentsContainer
from waeup.sirp.students.accommodation import StudentAccommodation
from waeup.sirp.utils.helpers import attrs_to_fields
from waeup.sirp.students.utils import generate_student_id

class Student(grok.Container):
    """This is a student container for the various objects
    owned by students.
    """
    grok.implements(IStudent, IStudentNavigation)
    grok.provides(IStudent)

    def __init__(self):
        super(Student, self).__init__()
        # The site doesn't exist in unit tests
        try:
            students = grok.getSite()['students']
            self.student_id = generate_student_id(students,'?')
        except TypeError:
            self.student_id = u'Z654321'
        self.password = None
        return

    def loggerInfo(self, ob_class, comment=None):
        target = self.__name__
        return grok.getSite()['students'].logger_info(ob_class,target,comment)

    @property
    def display_fullname(self):
        middlename = getattr(self, 'middlename', None)
        sirp_utils = getUtility(ISIRPUtils)
        return sirp_utils.fullname(self.firstname, self.lastname, middlename)

    @property
    def fullname(self):
        middlename = getattr(self, 'middlename', None)
        if middlename:
            return '%s-%s-%s' % (self.firstname.lower(),
                middlename.lower(), self.lastname.lower())
        else:
            return '%s-%s' % (self.firstname.lower(), self.lastname.lower())

    @property
    def state(self):
        state = IWorkflowState(self).getState()
        return state

    @property
    def translated_state(self):
        state = registration_states_vocab.getTermByToken(
            self.state).title
        return state

    @property
    def history(self):
        history = IObjectHistory(self)
        return history

    def getStudent(self):
        return self

    @property
    def certcode(self):
        cert = getattr(self.get('studycourse', None), 'certificate', None)
        if cert is not None:
            return cert.code
        return

    @property
    def faccode(self):
        cert = getattr(self.get('studycourse', None), 'certificate', None)
        if cert is not None:
            return cert.__parent__.__parent__.__parent__.code
        return

    @property
    def depcode(self):
        cert = getattr(self.get('studycourse', None), 'certificate', None)
        if cert is not None:
            return cert.__parent__.__parent__.code
        return

    @property
    def current_session(self):
        session = getattr(self.get('studycourse', None), 'current_session', None)
        return session

    @property
    def current_mode(self):
        certificate = getattr(self.get('studycourse', None), 'certificate', None)
        if certificate is not None:
            return certificate.study_mode
        return

# Set all attributes of Student required in IStudent as field
# properties. Doing this, we do not have to set initial attributes
# ourselves and as a bonus we get free validation when an attribute is
# set.
Student = attrs_to_fields(Student)

class StudentFactory(grok.GlobalUtility):
    """A factory for students.
    """
    grok.implements(IFactory)
    grok.name(u'waeup.Student')
    title = u"Create a new student.",
    description = u"This factory instantiates new student instances."

    def __call__(self, *args, **kw):
        return Student()

    def getInterfaces(self):
        return implementedBy(Student)

@grok.subscribe(IStudent, grok.IObjectAddedEvent)
def handle_student_added(student, event):
    """If a student is added all subcontainers are automatically added
    and the transition create is fired. The latter produces a logging message.
    """
    reg_state = IWorkflowState(student).getState()
    if reg_state == CLEARANCE:
        student.clearance_locked = False
    else:
        student.clearance_locked = True
    studycourse = StudentStudyCourse()
    student['studycourse'] = studycourse
    payments = StudentPaymentsContainer()
    student['payments'] = payments
    accommodation = StudentAccommodation()
    student['accommodation'] = accommodation
    # Assign global student role for new student
    account = IUserAccount(student)
    account.roles = ['waeup.Student']
    # Assign local StudentRecordOwner role
    role_manager = IPrincipalRoleManager(student)
    role_manager.assignRoleToPrincipal(
        'waeup.local.StudentRecordOwner', student.student_id)
    if reg_state is None:
        IWorkflowInfo(student).fireTransition('create')
    return

@grok.subscribe(IStudent, grok.IObjectRemovedEvent)
def handle_student_removed(student, event):
    """If a student is removed a message is logged.
    """
    comment = 'Student record removed'
    target = student.student_id
    try:
        grok.getSite()['students'].logger.info('%s - %s' % (
            target, comment))
    except KeyError:
        # If we delete an entire university instance there won't be
        # a students subcontainer
        return
    return

#: The file id marker for student files
STUDENT_FILE_STORE_NAME = 'file-student'

class StudentFileNameChooser(grok.Adapter):
    """A file id chooser for :class:`Student` objects.

    `context` is an :class:`Student` instance.

    The :class:`StudentImageNameChooser` can build/check file ids for
    :class:`Student` objects suitable for use with
    :class:`ExtFileStore` instances. The delivered file_id contains
    the file id marker for :class:`Student` object and the student id
    of the context student.

    This chooser is registered as an adapter providing
    :class:`waeup.sirp.interfaces.IFileStoreNameChooser`.

    File store name choosers like this one are only convenience
    components to ease the task of creating file ids for student
    objects. You are nevertheless encouraged to use them instead of
    manually setting up filenames for students.

    .. seealso:: :mod:`waeup.sirp.imagestorage`

    """
    grok.context(IStudent)
    grok.implements(IFileStoreNameChooser)

    def checkName(self, name=None, attr=None):
        """Check whether the given name is a valid file id for the context.

        Returns ``True`` only if `name` equals the result of
        :meth:`chooseName`.

        """
        return name == self.chooseName()

    def chooseName(self, attr, name=None):
        """Get a valid file id for student context.

        *Example:*

        For a student with student id ``'A123456'`` and
        with attr ``'nice_image.jpeg'`` stored in
        the students container this chooser would create:

          ``'__file-student__students/A/A123456/nice_image_A123456.jpeg'``

        meaning that the nice image of this applicant would be
        stored in the site-wide file storage in path:

          ``students/A/A123456/nice_image_A123456.jpeg``

        """
        basename, ext = os.path.splitext(attr)
        stud_id = self.context.student_id
        marked_filename = '__%s__%s/%s/%s_%s%s' % (
            STUDENT_FILE_STORE_NAME, stud_id[0], stud_id, basename, stud_id, ext)
        return marked_filename


class StudentFileStoreHandler(DefaultFileStoreHandler, grok.GlobalUtility):
    """Student specific file handling.

    This handler knows in which path in a filestore to store student
    files and how to turn this kind of data into some (browsable)
    file object.

    It is called from the global file storage, when it wants to
    get/store a file with a file id starting with
    ``__file-student__`` (the marker string for student files).

    Like each other file store handler it does not handle the files
    really (this is done by the global file store) but only computes
    paths and things like this.
    """
    grok.implements(IFileStoreHandler)
    grok.name(STUDENT_FILE_STORE_NAME)

    def pathFromFileID(self, store, root, file_id):
        """All student files are put in directory ``students``.
        """
        marker, filename, basename, ext = store.extractMarker(file_id)
        sub_root = os.path.join(root, 'students')
        return super(StudentFileStoreHandler, self).pathFromFileID(
            store, sub_root, basename)

    def createFile(self, store, root, filename, file_id, file):
        """Create a browsable file-like object.
        """
        # call super method to ensure that any old files with
        # different filename extension are deleted.
        file, path, file_obj =  super(
            StudentFileStoreHandler, self).createFile(
            store, root,  filename, file_id, file)
        return file, path, SIRPImageFile(
            file_obj.filename, file_obj.data)
